using HackathonPreparing.ApiService.Features.AuthFeature;
using HackathonPreparing.ApiService.Features.AuthFeature.EfCore;
using HackathonPreparing.ApiService.Features.DevFeature;
using HackathonPreparing.ApiService.Features.WeatherForecastFeature;
using HackathonPreparing.ApiService.Features.WeatherForecastFeature.EfCore;
// using HackathonPreparing.ServiceDefaults;
using Microsoft.OpenApi.Models;

var builder = WebApplication.CreateBuilder(args);

builder.Services.AddEndpointsApiExplorer();
builder.Services.AddSwaggerGen(c =>
{
    c.SchemaFilter<UserOpenApiSchemeFilter>();
    c.AddSecurityDefinition("AccessToken", new OpenApiSecurityScheme {
        In = ParameterLocation.Header,
        Description = "can get in /login or /register",
        Name = "Authorization",
        Type = SecuritySchemeType.Http,
        BearerFormat = "JWT",
        Scheme = "bearer"
    });

    c.AddSecurityRequirement(new OpenApiSecurityRequirement{
        {
            new OpenApiSecurityScheme {
                Reference = new OpenApiReference{
                    Type = ReferenceType.SecurityScheme,
                    Id = "AccessToken"
                }
            },
            new string[]{}
        }
    });
});

// Add service defaults & Aspire components.
//we dont need it now
//builder.AddServiceDefaults();

builder.Services.AddAuthorization();
builder.Services.AddAuthentication().AddBearerToken();

builder.Services
    .AddIdentityApiEndpoints<User>()
    .AddEntityFrameworkStores<UserContext>();

// Add services to the container.
builder.Services.AddProblemDetails();

builder.AddDbContextWithDefaultConfiguration<WeatherForecastContext>();
builder.AddDbContextWithDefaultConfiguration<UserContext>();

var app = builder.Build();

app.UsePathBase("/api");
app.UseRouting();

app.MapGet("/test", () => {});

if (app.Environment.IsDevelopment())
{
    var logger = app.Services.GetRequiredService<ILogger<Program>>();
    logger.LogWarning("!!!!!!! Call /ensure-created enpoint or /migrate if no db !!!!!!!!");
    app.UseSwagger();
    app.UseSwaggerUI();
    app.AddDbHelpersEndpoints();
}

//Configure the HTTP request pipeline.
app.UseExceptionHandler();

app.UseAuthentication();
app.UseAuthorization();

app.AddWeatherForecastEndpoints();
// app.MapDefaultEndpoints();

app.MapGroup("user")
    .MapIdentityApi<User>()
    .WithTags("user");

app.Run();
